package main

import (
    "fmt"
    "os"

    "github.com/aws/aws-sdk-go/aws"
    "github.com/aws/aws-sdk-go/aws/credentials"
    "github.com/aws/aws-sdk-go/aws/session"
    "github.com/aws/aws-sdk-go/service/s3"
    "github.com/aws/aws-sdk-go/service/s3/s3manager"
)

func main() {

	bucket := "c123456"
	domain := "https://c123456.parspack.net"
	accessKey := "YOUR_ACCESS_KEY"
	secretKey := "YOUR_SECRET_KEY"

    sess, err := session.NewSession(&aws.Config{
        Credentials: credentials.NewStaticCredentials(accessKey, secretKey, ""),
        Region: aws.String("default"),
        Endpoint: aws.String(domain),
            S3ForcePathStyle: aws.Bool(true),
    })

    if err != nil {
        exitErrorf("Unable to authenticate for bucket %q, %v", bucket, err)
    }

    svc := s3.New(sess, &aws.Config{
        Region:   aws.String("default"),
        Endpoint: aws.String(domain),
            S3ForcePathStyle: aws.Bool(true),
    })

    command := os.Args[1]

    switch command {
        case "list":
            // Get the list of items
            resp, err := svc.ListObjectsV2(&s3.ListObjectsV2Input{Bucket: aws.String(bucket)})
            if err != nil {
                exitErrorf("Unable to list items in bucket %q, %v", bucket, err)
            }

            for _, item := range resp.Contents {
                fmt.Println("Name:         ", *item.Key)
                fmt.Println("Size:         ", *item.Size)
                fmt.Println("")
            }

            fmt.Println(len(resp.Contents), "files")
            fmt.Println("")
        case "put":
            filename := "./parspack.png"
            file, err := os.Open(filename)
            if err != nil {
                exitErrorf("Unable to open file %q, %v", err)
            }
            defer file.Close()

            uploader := s3manager.NewUploader(sess)
            _, err = uploader.Upload(&s3manager.UploadInput{
                Bucket: aws.String(bucket),
                Key: aws.String(filename),
                Body: file,
            })
            if err != nil {
                // Print the error and exit.
                exitErrorf("Unable to upload %q to %q, %v", filename, bucket, err)
            }
            fmt.Println("Uploaded file: ",filename)
        case "get":
            item := "parspack-downloaded.png"

            file, err := os.Create(item)
            if err != nil {
                exitErrorf("Unable to open file %q, %v", item, err)
            }
            defer file.Close()

            downloader := s3manager.NewDownloader(sess)
            numBytes, err := downloader.Download(file,
                &s3.GetObjectInput{
                    Bucket: aws.String(bucket),
                    Key:    aws.String("parspack.png"),
                })
            if err != nil {
                exitErrorf("Unable to download item %q, %v", item, err)
            }

            fmt.Println("Downloaded", file.Name(), numBytes, "bytes")
        case "delete":
            obj := "parspack.png"
            _, err = svc.DeleteObject(&s3.DeleteObjectInput{Bucket: aws.String(bucket), Key: aws.String(obj)})
            if err != nil {
                exitErrorf("Unable to delete object %q from bucket %q, %v", obj, bucket, err)
            }

            err = svc.WaitUntilObjectNotExists(&s3.HeadObjectInput{
                Bucket: aws.String(bucket),
                Key:    aws.String(obj),
            })
            if err != nil {
                exitErrorf("Error occurred while waiting for object %q to be deleted, %v", obj, err)
            }

            fmt.Printf("File %q deleted\n", obj)
        default:
            fmt.Println("Invalid command")
   }
}

func exitErrorf(msg string, args ...interface{}) {
    fmt.Fprintf(os.Stderr, msg+"\n", args...)
    os.Exit(1)
}